
#ifdef GL_ES
precision highp float;
#endif

// Vertex inputs
attribute vec4 inPosition;  
attribute lowp vec4 inColor;
attribute mediump vec2 inTexCoord;

// Projection matrix
uniform mat4 modelViewProjectionMatrix;
uniform vec4 modelUpVec;
uniform vec4 userParm0;	//@from MASIL

// Fog
uniform mediump vec4 fogParams;
uniform mediump vec4 mirrorParams;
varying lowp vec4 varFog;


// Vertex outputs
varying lowp vec4 varColor;
varying lowp vec4 varUserParm;
varying mediump vec2 varTexCoord;

void main (void) 
{
// Calculate position
    vec4 pos = inPosition;
    pos.w = 1.0;
	gl_Position	= modelViewProjectionMatrix * pos;
    
// Set the color
    varColor = inColor;
	varUserParm = userParm0;

// Is it in fog?
    mediump float mirrorHeight = mirrorParams.x;
    mediump float mirrorMul = mirrorParams.y;
    mediump float mirrorAdd = mirrorParams.z;
    mediump float vertFog = dot(pos, modelUpVec);
    vertFog = mirrorHeight - vertFog;
    vertFog *= mirrorMul;
    vertFog += mirrorAdd;
    vertFog = clamp(vertFog, 0.0, 255.0);
    
    mediump float fogStart = fogParams.x;
    mediump float fogMul = fogParams.y;
    mediump float fog = (gl_Position.w - fogStart) * fogMul;
    fog = clamp(fog, 0.0, 255.0);
    fog += vertFog;
    fog = fog / 255.0;
    varFog.a = clamp(fog, 0.0, 1.0);

// Set the texture coordinates
    varTexCoord.x = inTexCoord.x;
    varTexCoord.y = 1.0-inTexCoord.y;
}

