
#ifdef GL_ES
precision highp float;
#endif

// Vertex inputs
attribute vec4 inPosition;  
attribute vec4 inColor;
attribute vec4 inNormal;
attribute vec2 inTexCoord;

// projection matrix
uniform mat4 modelViewProjectionMatrix;

#ifdef SHADER_USE_LIGHTS
uniform vec4 lightPosAndReach1;
uniform vec4 lightColAndCone1;
uniform vec4 lightDirection1;
#if (SHADER_NUM_LIGHTS > 1)
uniform vec4 lightPosAndReach2;
uniform vec4 lightColAndCone2;
uniform vec4 lightDirection2;
#endif
#if (SHADER_NUM_LIGHTS > 2)
uniform vec4 lightPosAndReach3;
uniform vec4 lightColAndCone3;
uniform vec4 lightDirection3;
#endif
#if (SHADER_NUM_LIGHTS > 3)
uniform vec4 lightPosAndReach4;
uniform vec4 lightColAndCone4;
uniform vec4 lightDirection4;
#endif
#endif

// Vertex outputs
varying vec4 varColor;
varying vec2 varTexCoord;

#ifdef SHADER_USE_LIGHTS
vec4 ProcessLight( vec4 zPosAndReach, vec4 zColAndCone, vec4 zDirection )
{
    vec4 diff = inPosition - zPosAndReach;
    float len = length(diff.xyz);
    if( len <= zPosAndReach.w )
    {
        float scale = len / zPosAndReach.w;
        
        float ang = dot( diff.xyz, inNormal.xyz );
        ang = clamp( ang/len, 0.0, 1.0 );
        scale = scale * (1.0-ang);
        
        vec4 col = mix( vec4(0.0, 0.0, 0.0, 0.0), zColAndCone, scale );
        col.a = 0.0;
        
        return col;
    }
    return vec4(0.0, 0.0, 0.0, 0.0);
}
#endif

void main (void) 
{
	gl_Position	= modelViewProjectionMatrix * inPosition;
    varColor = inColor;
    
#ifdef SHADER_USE_LIGHTS
    varColor = varColor + ProcessLight( lightPosAndReach1, lightColAndCone1, lightDirection1 );
#if (SHADER_NUM_LIGHTS > 1)
    varColor = varColor + ProcessLight( lightPosAndReach2, lightColAndCone2, lightDirection2 );
#endif
#if (SHADER_NUM_LIGHTS > 2)
    varColor = varColor + ProcessLight( lightPosAndReach3, lightColAndCone3, lightDirection3 );
#endif
#if (SHADER_NUM_LIGHTS > 3)
    varColor = varColor + ProcessLight( lightPosAndReach4, lightColAndCone4, lightDirection4 );
#endif
    varColor = clamp(varColor, 0.0, 1.0);
#endif
    
    varTexCoord.x = inTexCoord.x;
    varTexCoord.y = 1.0-inTexCoord.y;
}
 