#ifdef GL_ES
precision highp float;
#endif

// Vertex inputs
attribute vec4 inPosition;  
attribute lowp vec4 inColor;
attribute vec4 inNormal;
attribute mediump vec2 inTexCoord;

// Projection matrix
uniform mat4 modelViewProjectionMatrix;
uniform vec4 userParm0;	//@from MASIL

// Tint
uniform lowp vec4 tint;

// Light(s)
#ifdef SHADER_USE_LIGHTS
uniform vec4 lightPosAndReach1;
uniform vec4 lightColAndCone1;
uniform vec4 lightDirAndUseNormal1;
#if (SHADER_NUM_LIGHTS > 1)
uniform vec4 lightPosAndReach2;
uniform vec4 lightColAndCone2;
uniform vec4 lightDirAndUseNormal2;
#endif
#if (SHADER_NUM_LIGHTS > 2)
uniform vec4 lightPosAndReach3;
uniform vec4 lightColAndCone3;
uniform vec4 lightDirAndUseNormal3;
#endif
#if (SHADER_NUM_LIGHTS > 3)
uniform vec4 lightPosAndReach4;
uniform vec4 lightColAndCone4;
uniform vec4 lightDirAndUseNormal4;
#endif
#endif

// Fog
#ifdef SHADER_USE_FOG
uniform mediump vec4 fogParams;
varying lowp vec4 varFog;
#endif


// Vertex outputs
varying lowp vec4 varColor;
varying lowp vec4 varUserParm;
varying mediump vec2 varTexCoord;

#ifdef SHADER_USE_LIGHTS
vec4 ProcessLight( vec4 zPosAndReach, vec4 zColAndCone, vec4 zDirAndUseNormal )
{
    vec4 LightDelta = zPosAndReach - inPosition;
    float squaredist = dot(LightDelta.xyz, LightDelta.xyz);

    if( squaredist < zPosAndReach.w )
    {
        float scale = 1.0 - (squaredist / zPosAndReach.w);
        
        float dist = sqrt(squaredist);
        
        // Calculate angle from normal and adjust scale accordingly - if allowed
        float ang = dot( LightDelta.xyz, inNormal.xyz );
        ang = clamp( ang/dist, 0.0, 1.0 );
        scale *= zDirAndUseNormal.w != 0.0 ? ang : 1.0;
        
        // Calculate the cone adjustment and adjust scale accordingly - if allowed
        float cone = -dot( zDirAndUseNormal.xyz, LightDelta.xyz );
        cone = (cone / dist) -1.0;
        cone *= zColAndCone.w;
        cone += 1.0;
        cone = clamp( cone, 0.0, 1.0 );
        scale *= zColAndCone.w > 0.0 ? cone : 1.0;
        
        // Blend between no light and maximum light
        vec4 col = mix( vec4(0.0, 0.0, 0.0, 0.0), zColAndCone, scale );
        col.a = 0.0;
        
        return col;
    }
    return vec4(0.0, 0.0, 0.0, 0.0);
}
#endif

void main (void) 
{
// Calculate position
	gl_Position	= modelViewProjectionMatrix * inPosition;
    
// Set the color
    varColor = inColor;
	varUserParm = userParm0;
    
// Add any lights
//#ifdef SHADER_USE_LIGHTS
////    varColor = vec4(0.0,0.0,0.0,0.0);
//    varColor += ProcessLight( lightPosAndReach1, lightColAndCone1, lightDirAndUseNormal1 );
//#if (SHADER_NUM_LIGHTS > 1)
//    varColor += ProcessLight( lightPosAndReach2, lightColAndCone2, lightDirAndUseNormal2 );
//#endif
//#if (SHADER_NUM_LIGHTS > 2)
//    varColor += ProcessLight( lightPosAndReach3, lightColAndCone3, lightDirAndUseNormal3 );
//#endif
//#if (SHADER_NUM_LIGHTS > 3)
//    varColor += ProcessLight( lightPosAndReach4, lightColAndCone4, lightDirAndUseNormal4 );
//#endif
//#endif

// Add any tinting
    varColor += tint;
    
// ..and clamp the color
    varColor = clamp(varColor, 0.0, 1.0);// * userParm0.xyzw;	//@from MASIL
    
// Is it in fog?
#ifdef SHADER_USE_FOG
    mediump float fogStart = fogParams.x;
    mediump float fogMul = fogParams.y;
    mediump float fog = (gl_Position.w - fogStart) * fogMul;
    fog = clamp(fog, 0.0, 255.0);
    fog += inNormal.w;
    fog = fog / 255.0;
    varFog.a = clamp(fog, 0.0, 1.0);
#endif

// Set the texture coordinates
    varTexCoord.x = inTexCoord.x;
    varTexCoord.y = 1.0-inTexCoord.y;
} 
  