//@built-in
/////////////////////////////////////////////////////
precision highp float; 
precision highp int;

#define float2 vec2
#define float3 vec3
#define float4 vec4
#define matrix mat4
#define matrix33 mat3
#define samplerCB samplerCube
#define textureCB textureCube

#define saturate(_v) clamp(_v, 0.0, 1.0)
#define lerp(_a,_b,_r) mix(_a,_b,_r)


//@attribe
/////////////////////////////////////////////////////
#ifdef VShader
attribute float4 locP;
attribute float4 locN;
attribute float4 locC; 
attribute float2 locT; 
#endif


//@constant
/////////////////////////////////////////////////////
uniform highp mat4 c_mat0;	
uniform highp mat4 c_mat1;
uniform highp mat4 c_mat2;
uniform mediump float4 c_parm0;
uniform mediump float4 c_parm1;
uniform mediump float4 c_parm2;
uniform mediump float4 c_parm3;
uniform mediump float4 c_parm4;
uniform mediump float4 c_parm5;
uniform mediump float4 c_parm6;
uniform mediump float4 c_parm7;
uniform mediump float4 c_parm8;
uniform mediump float4 c_parm9;
uniform mediump float4 c_parm10;
uniform mediump float4 c_parm11;
uniform mediump float4 c_parm12;
uniform mediump float4 c_parm13;
uniform mediump float4 c_parm14;
uniform mediump float4 c_parm15;

#ifdef VShader
#define matW			c_mat0
#define matVP			c_mat1
#define matWVP		c_mat2
#endif

#ifdef PShader
uniform sampler2D c_samp0; 
uniform sampler2D c_samp1; 
uniform samplerCB c_cube; 
#endif


//@varying
/////////////////////////////////////////////////////
#define vs_out_p	gl_Position
#define ps_out_c	gl_FragColor

varying mediump float4 v_array0;	
varying mediump float4 v_array1;
varying mediump float4 v_array2;
varying mediump float4 v_array3;	
varying mediump float4 v_array4;	
varying mediump float4 v_array5;
varying mediump float4 v_array6;

	#define vs_out_0 	v_array0
	#define vs_out_1 	v_array1
	#define vs_out_2 	v_array2
	#define vs_out_3 	v_array3
	#define vs_out_4 	v_array4
	#define vs_out_5 	v_array5
	#define vs_out_6 	v_array6
	#define vs_out_7 	v_array7

	#define ps_in_0 	v_array0
	#define ps_in_1 	v_array1
	#define ps_in_2 	v_array2
	#define ps_in_3		v_array3
	#define ps_in_4 	v_array4
	#define ps_in_5 	v_array5
	#define ps_in_6 	v_array6
	#define ps_in_7 	v_array7



//@base
/////////////////////////////////////////////////////
#ifdef Base
#define UserColor		c_parm0
#define GrayRatio		c_parm1.x
#endif

#ifdef ShwScr
#define UserColor		c_parm0
#define GrayRatio		c_parm1.x
#endif


//@lynx
/////////////////////////////////////////////////////
#ifdef Lynx
	//c0: xyz:EyePos, w:GrayRatio
	//c1: xyz:SunDir, w:Ambient
	//c2: xyz:SunColor, w:LambertPow
	//c3: x:RimIntensity, y:RimBand, z:SpecIntensity, w:SpecPow
	//c4: xyz:IlluColor, w:ReflRatio
	//c5: xyzw:UserColor
	//c6: pointlit#0,		xyz:pos, w:range
	//c7: pointlit#1,		xyz:pos, w:range
	//c8: pointlit#2,		xyz:pos, w:range
	//c9: pointlit#3,		xyz:pos, w:range
	//c10: pointlit#0,	xyz:clr, w:pow
	//c11: pointlit#1,	xyz:clr, w:pow
	//c12: pointlit#2,	xyz:clr, w:pow
	//c13: pointlit#3,	xyz:clr, w:pow

	#define EyePos					c_parm0.xyz			//one-time
	#define SunDir					c_parm1.xyz	
	#define SunColor				c_parm2.xyz	
	#define Ambient					c_parm1.w		
	#define LambertPow			c_parm2.w		
	#define RimIntensity		c_parm3.x		
	#define RimBand					c_parm3.y		
	#define SpecIntensity		c_parm3.z		
	#define SpecPow					c_parm3.w		
	#define IlluColor				c_parm4.xyz			//every model
	#define	UserColor				c_parm5.xyzw		//"
	#define ReflRatio				c_parm4.w				//"
	#define GrayRatio				c_parm0.w

	#define PointLightPos0	c_parm6		
	#define PointLightPos1	c_parm7		
	#define PointLightPos2	c_parm8		
	#define PointLightPos3	c_parm9		
	#define PointLightClr0	c_parm10		
	#define PointLightClr1	c_parm11		
	#define PointLightClr2	c_parm12		
	#define PointLightClr3	c_parm13		
	
#endif



//@functions
/////////////////////////////////////////////////////
float calc_distance (float3 spos, float3 dpos) {
	float4 tpos = float4(spos, 1.0) - float4(dpos, 1.0);
	return length(tpos);
}

float get_dis_alpha (float dist, float near_dist, float far_dist) {
	return saturate ( (far_dist - dist) / (far_dist - near_dist)  );
}

float4 calc_hemisphere (float3 vN, float3 vSky, float3 grwC, float3 skyC, float amb) {
	float dt = (dot(vN.xyz, vSky)+1.0)/2.0;
	return float4 ( lerp (grwC.xyz, skyC.xyz, dt)*amb, dt );
}

float calc_diffuse (float3 vL, float3 vN, float power, float amb) {
	return pow ( amb+max(0.0, dot(vL,vN))*(1.0-amb), power );
}

float calc_rimlight (float3 vL, float3 vN, float3 vE, float intensity, float band) {
  float NV = saturate ( dot (vN, vE) );
  float rimpow = 1.0 + max( 0.0, dot(-vE, vL) );						
	return ( smoothstep ( -1.0+band, 1.0, 1.0-NV ) * rimpow * intensity );
}

float calc_specular (float3 vL, float3 vN, float3 vE, float intensity, float power) {
  float3 vH = normalize (vE + vL);
	return (intensity * pow (saturate(dot(vH, vN)), power ));
}

