//@vary
//v0: xyz:diff, w:---
//v1: xyz:spec, w:---
//v2: xyz:refluv, w:refl_ratio
//v3: xy:uv, zw:---
//v4: xyz:glow
//v5: xyz:point_lit

#define GlowPerSpec 1.0

float3 calc_pointlit (float3 wP, float3 wN, float4 litPos, float4 litClr)
{
	float dis = calc_distance (wP.xyz, litPos.xyz) / litPos.w;
	float att = 1.0 - saturate(dis);
	att = pow (att, 2.0+litClr.w*5.0);

	float3 wL = normalize (litPos.xyz - wP.xyz);
	float dt = saturate (dot (wN, wL));

	return (litClr.xyz * dt * att);
}


void main ()
{
#ifdef LynxDetail

	float3 wP = (matW * locP).xyz;
	float3 wN = normalize(matW * float4(locN.xyz, 0.0)).xyz;
	float3 wE = normalize (EyePos - wP.xyz).xyz;
	float3 wS = SunDir;
	
	float4 ambc = float4(1,1,1,1) * Ambient;
	float sun_diff = calc_diffuse (wS, wN, LambertPow, 0.5);//Ambient);
	float sun_rim = calc_rimlight (wS, wN, wE, RimIntensity, RimBand);
	float sun_spec = calc_specular (wS, wN, wE, SpecIntensity, SpecPow);

	float3	plit =  calc_pointlit (wP, wN, PointLightPos0, PointLightClr0);
					plit += calc_pointlit (wP, wN, PointLightPos1, PointLightClr1);
					plit += calc_pointlit (wP, wN, PointLightPos2, PointLightClr2);
					plit += calc_pointlit (wP, wN, PointLightPos3, PointLightClr3);
	//float3 plit = float3 (0,0,0);

	float3 diff = (ambc.xyz + (sun_diff + sun_rim) * SunColor);// + plit.xyz;
	float3 spec = (sun_spec * SunColor);// + plit.xyz;
	//float3 diff = (ambc.xyz + (sun_diff + sun_rim) * SunColor) + plit.xyz + (sun_spec * SunColor);

	vs_out_p = matWVP * locP;
	vs_out_0 = float4 (saturate (diff), 1.0);
	vs_out_1 = float4 (saturate (spec), 0.0);
	vs_out_2 = float4 (reflect(-wE, wN.xzy), ReflRatio);
	vs_out_3 = float4 (locT.x, 1.0-locT.y, 0.0, 0.0);
	vs_out_4 = float4 (0,0,0,0);
	vs_out_5 = float4 (plit.xyz, 0.0);

#endif

#ifdef LynxSimple

	float3 wP = (matW * locP).xyz;
	float3 wN = normalize(matW * float4(locN.xyz, 0.0)).xyz;
	float3 wE = normalize (EyePos - wP.xyz).xyz;
	float3 wS = SunDir;
	
	float4 ambc = float4(1,1,1,1) * Ambient;
	float sun_diff = calc_diffuse (wS, wN, LambertPow, 0.5);//Ambient);
	float sun_rim = 0.0;//calc_rimlight (wS, wN, wE, RimIntensity, RimBand);
	float sun_spec = calc_specular (wS, wN, wE, SpecIntensity, SpecPow);
		
	float3	plit =  calc_pointlit (wP, wN, PointLightPos0, PointLightClr0);
					plit += calc_pointlit (wP, wN, PointLightPos1, PointLightClr1);
					plit += calc_pointlit (wP, wN, PointLightPos2, PointLightClr2);
					plit += calc_pointlit (wP, wN, PointLightPos3, PointLightClr3);

	float3 diff = (ambc.xyz + (sun_diff + sun_rim) * SunColor) + plit.xyz + (sun_spec * SunColor);
	diff *= UserColor.xyz;

	vs_out_p = matWVP * locP;
	vs_out_0 = float4 (saturate (diff), 1.0);
	vs_out_1 = float4 (0,0,0,0);
	vs_out_2 = float4 (0,0,0,0);//reflect(-wE, wN.xzy), ReflRatio);
	vs_out_3 = float4 (locT.x, 1.0-locT.y, 0.0, 0.0);
	vs_out_4 = float4 (0,0,0,0);

#endif

#ifdef LynxMedium

	float3 wP = (matW * locP).xyz;
	float3 wN = normalize(matW * float4(locN.xyz, 0.0)).xyz;
	float3 wE = normalize (EyePos - wP.xyz).xyz;
	float3 wS = SunDir;
	
	float4 ambc = float4(1,1,1,1) * Ambient;
	float sun_diff = calc_diffuse (wS, wN, LambertPow, 0.5);//Ambient);
	float sun_rim = 0.0;//calc_rimlight (wS, wN, wE, RimIntensity, RimBand);
	float sun_spec = calc_specular (wS, wN, wE, SpecIntensity, SpecPow);
		
	float3	plit =  calc_pointlit (wP, wN, PointLightPos0, PointLightClr0);
					plit += calc_pointlit (wP, wN, PointLightPos1, PointLightClr1);
					plit += calc_pointlit (wP, wN, PointLightPos2, PointLightClr2);
					plit += calc_pointlit (wP, wN, PointLightPos3, PointLightClr3);

	float3 diff = (ambc.xyz + (sun_diff + sun_rim) * SunColor) + plit.xyz + (sun_spec * SunColor);
	diff *= UserColor.xyz;

	vs_out_p = matWVP * locP;
	vs_out_0 = float4 (saturate (diff), 1.0);
	vs_out_1 = float4 (0,0,0,0);
	vs_out_2 = float4 (0,0,0,0);//reflect(-wE, wN.xzy), ReflRatio);
	vs_out_3 = float4 (locT.x, 1.0-locT.y, 0.0, 0.0);
	vs_out_4 = float4 (0,0,0,0);

#endif
}



